/*----------------------------------------------------------------------- 

File Name: ArrayFactories.c

<lalVerbatim file="ArrayFactoriesCV">
Revision: $Id: ArrayFactories.m4,v 1.2 2000/06/03 21:36:34 jolien Exp $
</lalVerbatim>

-------------------------------------------------------------------------*/

/* <lalLaTeX>

\subsection{Module \texttt{ArrayFactories.c}}
\label{ss:ArrayFactories.c}

Create/destroy $\langle\mbox{datatype}\rangle$Array objects. 

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{ArrayFactoriesD}
\index{\verb&ZCreateArray()&}
\index{\verb&CCreateArray()&}
\index{\verb&DCreateArray()&}
\index{\verb&SCreateArray()&}
\index{\verb&I2CreateArray()&}
\index{\verb&I4CreateArray()&}
\index{\verb&I8CreateArray()&}
\index{\verb&U2CreateArray()&}
\index{\verb&U4CreateArray()&}
\index{\verb&U8CreateArray()&}
\index{\verb&CreateArray()&}
\index{\verb&ZDestroyArray()&}
\index{\verb&CDestroyArray()&}
\index{\verb&DDestroyArray()&}
\index{\verb&SDestroyArray()&}
\index{\verb&I2DestroyArray()&}
\index{\verb&I4DestroyArray()&}
\index{\verb&I8DestroyArray()&}
\index{\verb&U2DestroyArray()&}
\index{\verb&U4DestroyArray()&}
\index{\verb&U8DestroyArray()&}
\index{\verb&DestroyArray()&}

\subsubsection*{Description}

The \texttt{CreateArray} family of functions create a
$\langle\mbox{datatype}\rangle$\texttt{Array} of the appropriate dimensions.

The \texttt{DestroyArray} family of functions return the storage allocated by
the \texttt{CreateArray} functions to the system.

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
LALMalloc()
LALFree()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{ArrayFactoriesCV}}

</lalLaTeX> */

#include <string.h>
#include "LALStdlib.h"
#include "AVFactories.h"

/* <lalVerbatim file="ArrayFactoriesNRCSID"> */
NRCSID( ARRAYFACTORIESC, "$Id: ArrayFactories.m4,v 1.2 2000/06/03 21:36:34 jolien Exp $" );
/* </lalVerbatim> */
















/* <lalVerbatim file="ArrayFactoriesD"> */
void ZCreateArray ( Status *status, COMPLEX16Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "ZCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( COMPLEX16Array * ) LALMalloc ( sizeof( COMPLEX16Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( COMPLEX16 * ) LALMalloc( arrayDataSize*sizeof( COMPLEX16 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void ZDestroyArray ( Status *status, COMPLEX16Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "ZDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void CCreateArray ( Status *status, COMPLEX8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "CCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( COMPLEX8Array * ) LALMalloc ( sizeof( COMPLEX8Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( COMPLEX8 * ) LALMalloc( arrayDataSize*sizeof( COMPLEX8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void CDestroyArray ( Status *status, COMPLEX8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "CDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void DCreateArray ( Status *status, REAL8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "DCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL8Array * ) LALMalloc ( sizeof( REAL8Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL8 * ) LALMalloc( arrayDataSize*sizeof( REAL8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void DDestroyArray ( Status *status, REAL8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "DDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void SCreateArray ( Status *status, REAL4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "SCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL4Array * ) LALMalloc ( sizeof( REAL4Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL4 * ) LALMalloc( arrayDataSize*sizeof( REAL4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void SDestroyArray ( Status *status, REAL4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "SDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void I2CreateArray ( Status *status, INT2Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "I2CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT2Array * ) LALMalloc ( sizeof( INT2Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT2 * ) LALMalloc( arrayDataSize*sizeof( INT2 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void I2DestroyArray ( Status *status, INT2Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "I2DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void I4CreateArray ( Status *status, INT4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "I4CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT4Array * ) LALMalloc ( sizeof( INT4Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT4 * ) LALMalloc( arrayDataSize*sizeof( INT4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void I4DestroyArray ( Status *status, INT4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "I4DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void I8CreateArray ( Status *status, INT8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "I8CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT8Array * ) LALMalloc ( sizeof( INT8Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT8 * ) LALMalloc( arrayDataSize*sizeof( INT8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void I8DestroyArray ( Status *status, INT8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "I8DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void U2CreateArray ( Status *status, UINT2Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "U2CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT2Array * ) LALMalloc ( sizeof( UINT2Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT2 * ) LALMalloc( arrayDataSize*sizeof( UINT2 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void U2DestroyArray ( Status *status, UINT2Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "U2DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void U4CreateArray ( Status *status, UINT4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "U4CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT4Array * ) LALMalloc ( sizeof( UINT4Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT4 * ) LALMalloc( arrayDataSize*sizeof( UINT4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void U4DestroyArray ( Status *status, UINT4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "U4DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void U8CreateArray ( Status *status, UINT8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "U8CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT8Array * ) LALMalloc ( sizeof( UINT8Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT8 * ) LALMalloc( arrayDataSize*sizeof( UINT8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void U8DestroyArray ( Status *status, UINT8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "U8DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void CreateArray ( Status *status, REAL4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL4Array * ) LALMalloc ( sizeof( REAL4Array ) );
  ASSERT (*array, status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL4 * ) LALMalloc( arrayDataSize*sizeof( REAL4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















/* <lalVerbatim file="ArrayFactoriesD"> */
void DestroyArray ( Status *status, REAL4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

